@@ -1342,7 +1342,7 @@ on_video_button_press_event (BaconVideoWidget *bvw,
 			g_error_free (emb->error);
 			emb->error = NULL;
 		} else if (!gtk_widget_get_visible (GTK_WIDGET (menu))) {
-			g_message ("emitting signal");
+			g_message ("emitting signal button press");
 			g_signal_emit (emb, signals[BUTTON_PRESS], 0,
 				       event->time,
 				       event->button);
@@ -1781,8 +1781,8 @@ totem_embedded_construct (TotemEmbedded *emb,
 	g_assert (emb->menuxml);
 
 	popup_button = GTK_WIDGET (gtk_builder_get_object (emb->xml, "popup_button"));
-	gtk_menu_button_set_menu (GTK_MENU_BUTTON (popup_button),
-				  (GtkWidget *) gtk_builder_get_object (emb->menuxml, "menu"));
+	gtk_menu_button_set_popup (GTK_MENU_BUTTON (popup_button),
+				   (GtkWidget *) gtk_builder_get_object (emb->menuxml, "menu"));
 	gtk_menu_button_set_direction (GTK_MENU_BUTTON (popup_button), GTK_ARROW_UP);
 
 	emb->volume = GTK_WIDGET (gtk_builder_get_object (emb->xml, "volume_button"));
