@@ -16,7 +16,7 @@ static void
 test_bvw_set_mrl (GtkWidget *bvw, const char *path)
 {
 	mrl = g_strdup (path);
-	bacon_video_widget_open (BACON_VIDEO_WIDGET (bvw), mrl, NULL, NULL);
+	bacon_video_widget_open (BACON_VIDEO_WIDGET (bvw), mrl, NULL);
 }
 
 static void
@@ -76,14 +76,11 @@ int main
 	GOptionGroup *baconoptiongroup;
 	GError *error = NULL;
 	GtkWidget *win, *bvw;
+	GtkSettings *gtk_settings;
 
 #ifdef GDK_WINDOWING_X11
 	XInitThreads ();
 #endif
-#if !GLIB_CHECK_VERSION (2, 31, 0)
-	g_thread_init (NULL);
-#endif
-	gdk_threads_init ();
 
 	context = g_option_context_new ("- Play audio and video inside a web browser");
 	baconoptiongroup = bacon_video_widget_get_option_group();
@@ -106,6 +103,9 @@ int main
 		return 1;
 	}
 
+	gtk_settings = gtk_settings_get_default ();
+	g_object_set (G_OBJECT (gtk_settings), "gtk-application-prefer-dark-theme", TRUE, NULL);
+
 	win = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 	gtk_window_set_default_size (GTK_WINDOW (win), 500, 500);
 	g_signal_connect (G_OBJECT (win), "destroy",
