@@ -44,7 +44,9 @@
 #include <totem-dirs.h>
 #include <totem.h>
 
-#include <video-utils.h>
+#include <totem-time-helpers.h>
+
+#include "totem-search-entry.h"
 
 #define TOTEM_TYPE_GRILO_PLUGIN                                         \
 	(totem_grilo_plugin_get_type ())
@@ -69,7 +71,7 @@
 
 #define BROWSE_FLAGS          (GRL_RESOLVE_FAST_ONLY | GRL_RESOLVE_IDLE_RELAY)
 #define PAGE_SIZE             50
-#define THUMB_SEARCH_SIZE     128
+#define THUMB_SEARCH_SIZE     256
 #define THUMB_BROWSE_SIZE     32
 #define SCROLL_GET_MORE_LIMIT 0.8
 
@@ -98,7 +100,7 @@ typedef struct {
 	GHashTable *cache_thumbnails;
 
 	/* Search related information */
-	GrlMediaSource *search_source;
+	GrlSource *search_source;
 	guint search_id;
 	gint search_page;
 	gint search_remaining;
@@ -111,7 +113,7 @@ typedef struct {
 	/* Search widgets */
 	GtkWidget *search_entry;
 	GtkTreeModel *search_results_model;
-	GtkTreeModel *search_sources_model;
+	GHashTable *search_sources_ht;
 	GtkWidget *search_sources_list;
 	GtkWidget *search_results_view;
 
@@ -155,7 +157,7 @@ enum {
 };
 
 static void play (TotemGriloPlugin *self,
-                  GrlMediaSource *source,
+                  GrlSource *source,
                   GrlMedia *media,
                   gboolean resolve_url);
 
@@ -393,18 +395,18 @@ show_sources (TotemGriloPlugin *self)
 {
 	GList *sources, *source;
 	GtkTreeIter iter;
-	GrlPluginRegistry *registry;
+	GrlRegistry *registry;
 	const gchar *name;
 	GdkPixbuf *icon;
 
-	registry = grl_plugin_registry_get_default ();
-	sources = grl_plugin_registry_get_sources_by_operations (registry,
-	                                                         GRL_OP_BROWSE,
-	                                                         FALSE);
+	registry = grl_registry_get_default ();
+	sources = grl_registry_get_sources_by_operations (registry,
+	                                                  GRL_OP_BROWSE,
+	                                                  FALSE);
 
 	for (source = sources; source ; source = g_list_next (source)) {
 		icon = load_icon (self, ICON_BOX, THUMB_BROWSE_SIZE);
-		name = grl_metadata_source_get_name (GRL_METADATA_SOURCE (source->data));
+		name = grl_source_get_name (source->data);
 		gtk_tree_store_append (GTK_TREE_STORE (self->priv->browser_model), &iter, NULL);
 		gtk_tree_store_set (GTK_TREE_STORE (self->priv->browser_model),
 		                    &iter,
@@ -422,7 +424,7 @@ show_sources (TotemGriloPlugin *self)
 }
 
 static void
-browse_cb (GrlMediaSource *source,
+browse_cb (GrlSource *source,
            guint browse_id,
            GrlMedia *media,
            guint remaining,
@@ -508,28 +510,37 @@ browse_cb (GrlMediaSource *source,
 static void
 browse (TotemGriloPlugin *self,
         GtkTreePath *path,
-        GrlMediaSource *source,
+        GrlSource *source,
         GrlMedia *container,
         gint page)
 {
 	if (source != NULL) {
-		BrowseUserData *bud = g_slice_new (BrowseUserData);
+		BrowseUserData *bud;
+		GrlOperationOptions *default_options;
+
+		default_options = grl_operation_options_new (NULL);
+		grl_operation_options_set_flags (default_options, BROWSE_FLAGS);
+		grl_operation_options_set_skip (default_options, (page - 1) * PAGE_SIZE);
+		grl_operation_options_set_count (default_options, PAGE_SIZE);
+
+		bud = g_slice_new (BrowseUserData);
 		bud->totem_grilo = g_object_ref (self);
 		bud->ref_parent = gtk_tree_row_reference_new (self->priv->browser_model, path);
-		grl_media_source_browse (source,
-		                         container,
-		                         browse_keys (),
-		                         (page - 1) * PAGE_SIZE, PAGE_SIZE,
-		                         BROWSE_FLAGS,
-		                         browse_cb,
-		                         bud);
+		grl_source_browse (source,
+		                   container,
+		                   browse_keys (),
+		                   default_options,
+		                   browse_cb,
+		                   bud);
+
+		g_object_unref (default_options);
 	} else {
 		show_sources (self);
 	}
 }
 
 static void
-resolve_url_cb (GrlMediaSource *source,
+resolve_url_cb (GrlSource *source,
                 guint op_id,
                 GrlMedia *media,
                 gpointer user_data,
@@ -545,7 +556,7 @@ resolve_url_cb (GrlMediaSource *source,
 
 static void
 play (TotemGriloPlugin *self,
-      GrlMediaSource *source,
+      GrlSource *source,
       GrlMedia *media,
       gboolean resolve_url)
 {
@@ -554,8 +565,7 @@ play (TotemGriloPlugin *self,
 	url = grl_media_get_url (media);
 	if (url != NULL) {
 		totem_add_to_playlist_and_play (self->priv->totem, url,
-		                                grl_media_get_title (media),
-		                                TRUE);
+		                                grl_media_get_title (media));
 		return;
 	}
 
@@ -563,10 +573,10 @@ play (TotemGriloPlugin *self,
 	if (resolve_url) {
 		const GList *slow_keys;
 		GList *url_keys;
-		slow_keys = grl_metadata_source_slow_keys (GRL_METADATA_SOURCE (source));
-		if (g_list_find ((GList *) slow_keys, GRL_METADATA_KEY_URL) != NULL) {
+		slow_keys = grl_source_slow_keys (source);
+		if (g_list_find ((GList *) slow_keys, GINT_TO_POINTER (GRL_METADATA_KEY_URL)) != NULL) {
 			url_keys = grl_metadata_key_list_new (GRL_METADATA_KEY_URL, NULL);
-			grl_media_source_metadata (source, media, url_keys, 0, resolve_url_cb, self);
+			grl_source_resolve (source, media, url_keys, 0, resolve_url_cb, self);
 			g_list_free (url_keys);
 			return;
 		}
@@ -576,7 +586,7 @@ play (TotemGriloPlugin *self,
 }
 
 static void
-search_cb (GrlMediaSource *source,
+search_cb (GrlSource *source,
            guint search_id,
            GrlMedia *media,
            guint remaining,
@@ -635,24 +645,42 @@ search_cb (GrlMediaSource *source,
 static void
 search_more (TotemGriloPlugin *self)
 {
+	GrlOperationOptions *default_options;
+
+	default_options = grl_operation_options_new (NULL);
+	grl_operation_options_set_flags (default_options, BROWSE_FLAGS);
+	grl_operation_options_set_skip (default_options, (self->priv->search_page - 1) * PAGE_SIZE);
+	grl_operation_options_set_count (default_options, PAGE_SIZE);
+
 	gtk_widget_set_sensitive (self->priv->search_entry, FALSE);
 	self->priv->search_page++;
 	self->priv->search_remaining = PAGE_SIZE;
-	self->priv->search_id = grl_media_source_search (self->priv->search_source,
-	                                                 self->priv->search_text,
-	                                                 search_keys (),
-	                                                 (self->priv->search_page - 1) * PAGE_SIZE,
-	                                                 PAGE_SIZE,
-	                                                 BROWSE_FLAGS,
-	                                                 search_cb,
-	                                                 self);
+	if (self->priv->search_source != NULL) {
+		self->priv->search_id =
+			grl_source_search (self->priv->search_source,
+			                   self->priv->search_text,
+			                   search_keys (),
+			                   default_options,
+			                   search_cb,
+			                   self);
+	} else {
+		self->priv->search_id =
+			grl_multiple_search (NULL,
+			                     self->priv->search_text,
+			                     search_keys (),
+			                     default_options,
+			                     search_cb,
+			                     self);
+	}
+	g_object_unref (default_options);
+
 	if (self->priv->search_id == 0) {
 		search_cb (self->priv->search_source, 0, NULL, 0, self, NULL);
 	}
 }
 
 static void
-search (TotemGriloPlugin *self, GrlMediaSource *source, const gchar *text)
+search (TotemGriloPlugin *self, GrlSource *source, const gchar *text)
 {
 	gtk_list_store_clear (GTK_LIST_STORE (self->priv->search_results_model));
 	g_hash_table_remove_all (self->priv->cache_thumbnails);
@@ -664,52 +692,23 @@ search (TotemGriloPlugin *self, GrlMediaSource *source, const gchar *text)
 	search_more (self);
 }
 
-static void
-search_entry_changed_cb (GtkEntry *entry, TotemGriloPlugin *self)
-{
-	if (g_strcmp0 (gtk_entry_get_text (entry), "") == 0) {
-		g_object_set (G_OBJECT (entry),
-		              "secondary-icon-name", "edit-find-symbolic",
-		              "secondary-icon-activatable", FALSE,
-		              "secondary-icon-sensitive", FALSE,
-		              NULL);
-	} else {
-		g_object_set (G_OBJECT (entry),
-		              "secondary-icon-name", "edit-clear-symbolic",
-		              "secondary-icon-activatable", TRUE,
-		              "secondary-icon-sensitive", TRUE,
-		              NULL);
-	}
-}
-
 static void
 search_entry_activate_cb (GtkEntry *entry, TotemGriloPlugin *self)
 {
-	GtkTreeIter iter;
-	GrlMediaSource *source = NULL;
-	const gchar *text;
+	GrlRegistry *registry;
+	const char *id;
+	const char *text;
+	GrlSource *source;
 
-	if (gtk_widget_is_sensitive (self->priv->search_sources_list)) {
-		if (gtk_combo_box_get_active_iter (GTK_COMBO_BOX (self->priv->search_sources_list),
-		                                   &iter)) {
-			gtk_tree_model_get (self->priv->search_sources_model, &iter,
-			                    SEARCH_MODEL_SOURCES_SOURCE, &source,
-			                    -1);
-		}
-	}
+	id = totem_search_entry_get_selected_id (TOTEM_SEARCH_ENTRY (self->priv->search_entry));
+	g_return_if_fail (id != NULL);
+	registry = grl_registry_get_default ();
+	source = grl_registry_lookup_source (registry, id);
+	g_return_if_fail (source != NULL);
 
-	text = gtk_entry_get_text (GTK_ENTRY (self->priv->search_entry));
+	text = totem_search_entry_get_text (TOTEM_SEARCH_ENTRY (self->priv->search_entry));
+	g_return_if_fail (text != NULL);
 	search (self, source, text);
-
-	if (source != NULL) {
-		g_object_unref (source);
-	}
-}
-
-static void
-search_entry_clear_cb (GtkEntry *entry, TotemGriloPlugin *self)
-{
-	gtk_entry_set_text (entry, "");
 }
 
 static void
@@ -723,7 +722,7 @@ browser_activated_cb (GtkTreeView *tree_view,
 	GtkTreeModel *model;
 	GtkTreeIter iter;
 	GrlMedia *content;
-	GrlMediaSource *source;
+	GrlSource *source;
 	TotemGriloPlugin *self = TOTEM_GRILO_PLUGIN (user_data);
 
 	model = gtk_tree_view_get_model (tree_view);
@@ -769,9 +768,11 @@ browser_activated_cb (GtkTreeView *tree_view,
 }
 
 static void
-search_source_changed_cb (GtkComboBox *combo,
-			  TotemGriloPlugin *self)
+search_entry_source_changed_cb (GObject          *object,
+                                GParamSpec       *pspec,
+                                TotemGriloPlugin *self)
 {
+	/* FIXME: Do we actually want to do that? */
 	if (self->priv->search_id > 0) {
 		grl_operation_cancel (self->priv->search_id);
 		self->priv->search_id = 0;
@@ -786,7 +787,7 @@ search_activated_cb (GtkIconView *icon_view,
 {
 	GtkTreeModel *model;
 	GtkTreeIter iter;
-	GrlMediaSource *source;
+	GrlSource *source;
 	GrlMedia *content;
 
 	model = gtk_icon_view_get_model (icon_view);
@@ -808,9 +809,9 @@ search_activated_cb (GtkIconView *icon_view,
 }
 
 static gboolean
-source_is_blacklisted (GrlMediaSource *source)
+source_is_blacklisted (GrlSource *source)
 {
-	const gchar *id = grl_metadata_source_get_id (GRL_METADATA_SOURCE (source));
+	const gchar *id = grl_source_get_id (source);
 	const gchar **s = BLACKLIST_SOURCES;
 
 	while (*s) {
@@ -824,8 +825,8 @@ source_is_blacklisted (GrlMediaSource *source)
 }
 
 static void
-source_added_cb (GrlPluginRegistry *registry,
-                 GrlMediaSource *source,
+source_added_cb (GrlRegistry *registry,
+                 GrlSource *source,
                  gpointer user_data)
 {
 	const gchar *name;
@@ -836,16 +837,16 @@ source_added_cb (GrlPluginRegistry *registry,
 	GrlSupportedOps ops;
 
 	if (source_is_blacklisted (source)) {
-		grl_plugin_registry_unregister_source (registry,
-		                                       GRL_MEDIA_PLUGIN (source),
-		                                       NULL);
+		grl_registry_unregister_source (registry,
+		                                source,
+		                                NULL);
 		return;
 	}
 
 	self = TOTEM_GRILO_PLUGIN (user_data);
 	icon = load_icon (self, ICON_BOX, THUMB_BROWSE_SIZE);
-	name = grl_metadata_source_get_name (GRL_METADATA_SOURCE (source));
-	ops = grl_metadata_source_supported_operations (GRL_METADATA_SOURCE (source));
+	name = grl_source_get_name (source);
+	ops = grl_source_supported_operations (source);
 	if (ops & GRL_OP_BROWSE) {
 		description = g_markup_printf_escaped ("<b>%s</b>", name);
 		gtk_tree_store_append (GTK_TREE_STORE (self->priv->browser_model), &iter, NULL);
@@ -860,17 +861,12 @@ source_added_cb (GrlPluginRegistry *registry,
 		g_free (description);
 	}
 	if (ops & GRL_OP_SEARCH) {
-		gtk_list_store_append (GTK_LIST_STORE (self->priv->search_sources_model), &iter);
-		gtk_list_store_set (GTK_LIST_STORE (self->priv->search_sources_model),
-		                    &iter,
-		                    SEARCH_MODEL_SOURCES_SOURCE, source,
-		                    SEARCH_MODEL_SOURCES_NAME, name,
-		                    -1);
-		/* Select one of them */
-		if (gtk_combo_box_get_active (GTK_COMBO_BOX (self->priv->search_sources_list)) == -1) {
-			gtk_combo_box_set_active (GTK_COMBO_BOX (self->priv->search_sources_list), 0);
-			gtk_widget_set_sensitive (self->priv->search_entry, TRUE);
-		}
+		/* FIXME:
+		 * Handle tracker/filesystem specifically, so that we have a "local" entry here */
+		totem_search_entry_add_source (TOTEM_SEARCH_ENTRY (self->priv->search_entry),
+					       grl_source_get_id (source),
+					       grl_source_get_name (source),
+					       0); /* FIXME: Use correct priority */
 	}
 
 	if (icon != NULL) {
@@ -884,8 +880,8 @@ remove_browse_result (GtkTreeModel *model,
                       GtkTreeIter *iter,
                       gpointer user_data)
 {
-	GrlMediaSource *removed_source = GRL_MEDIA_SOURCE (user_data);
-	GrlMediaSource *model_source;
+	GrlSource *removed_source = GRL_SOURCE (user_data);
+	GrlSource *model_source;
 	gboolean same_source;
 
 	gtk_tree_model_get (model, iter,
@@ -903,40 +899,15 @@ remove_browse_result (GtkTreeModel *model,
 	return same_source;
 }
 
-static gboolean
-remove_searchable_source (GtkTreeModel *model,
-                          GtkTreePath *path,
-                          GtkTreeIter *iter,
-                          gpointer user_data)
-{
-	GrlMediaSource *removed_source = GRL_MEDIA_SOURCE (user_data);
-	GrlMediaSource *model_source;
-	gboolean same_source;
-
-	gtk_tree_model_get (model, iter,
-	                    MODEL_RESULTS_SOURCE, &model_source,
-	                    -1);
-
-	same_source = (model_source == removed_source);
-
-	if (same_source) {
-		gtk_list_store_remove (GTK_LIST_STORE (model), iter);
-	}
-
-	g_object_unref (model_source);
-
-	return same_source;
-}
-
 static void
-source_removed_cb (GrlPluginRegistry *registry,
-                   GrlMediaSource *source,
+source_removed_cb (GrlRegistry *registry,
+                   GrlSource *source,
                    gpointer user_data)
 {
 	GrlSupportedOps ops;
 	TotemGriloPlugin *self = TOTEM_GRILO_PLUGIN (user_data);
 
-	ops = grl_metadata_source_supported_operations (GRL_METADATA_SOURCE (source));
+	ops = grl_source_supported_operations (source);
 
 	/* Remove source and content from browse results */
 	if (ops & GRL_OP_BROWSE) {
@@ -948,36 +919,32 @@ source_removed_cb (GrlPluginRegistry *registry,
 	/* If current search results belongs to removed source, clear the results. In
 	   any case, remove the source from the list of searchable sources */
 	if (ops & GRL_OP_SEARCH) {
+		const char *id;
+
 		if (self->priv->search_source == source) {
 			gtk_list_store_clear (GTK_LIST_STORE (self->priv->search_results_model));
 			self->priv->search_source = NULL;
 		}
 
-		gtk_tree_model_foreach (self->priv->search_sources_model,
-		                        remove_searchable_source,
-		                        source);
-
-		/* Select one if the selected source was removed */
-		if (gtk_combo_box_get_active (GTK_COMBO_BOX (self->priv->search_sources_list)) == -1) {
-			gtk_combo_box_set_active (GTK_COMBO_BOX (self->priv->search_sources_list), 0);
-		}
+		id = grl_source_get_id (source);
+		totem_search_entry_remove_source (TOTEM_SEARCH_ENTRY (self->priv->search_entry), id);
 	}
 }
 
 static void
 load_grilo_plugins (TotemGriloPlugin *self)
 {
-	GrlPluginRegistry *registry;
+	GrlRegistry *registry;
 	GError *error = NULL;
 
-	registry = grl_plugin_registry_get_default ();
+	registry = grl_registry_get_default ();
 
 	g_signal_connect (registry, "source-added",
 	                  G_CALLBACK (source_added_cb), self);
 	g_signal_connect (registry, "source-removed",
 	                  G_CALLBACK (source_removed_cb), self);
 
-	if (grl_plugin_registry_load_all (registry, &error) == FALSE) {
+	if (grl_registry_load_all_plugins (registry, &error) == FALSE) {
 		g_warning ("Failed to load grilo plugins: %s", error->message);
 		g_error_free (error);
 	}
@@ -994,7 +961,7 @@ show_popup_menu (TotemGriloPlugin *self, GtkWidget *view, GdkEventButton *event)
 	GList *sel_list = NULL;
 	GtkTreeModel *model;
 	GtkTreeIter iter;
-	GrlMediaSource *source;
+	GrlSource *source;
 	const gchar *url = NULL;
 
 	if (view == self->priv->browser) {
@@ -1123,7 +1090,7 @@ get_more_browse_results_cb (GtkAdjustment *adjustment,
 	GtkTreePath *end_path;
 	GtkTreePath *parent_path;
 	GtkTreeIter iter;
-	GrlMediaSource *source;
+	GrlSource *source;
 	GrlMedia *container;
 	gint page;
 	gint remaining;
@@ -1225,21 +1192,10 @@ setup_sidebar_search (TotemGriloPlugin *self,
                       GtkBuilder *builder)
 {
 	self->priv->search_results_model = GTK_TREE_MODEL (gtk_builder_get_object (builder, "gw_search_store_results"));
-	self->priv->search_sources_model = GTK_TREE_MODEL (gtk_builder_get_object (builder, "gw_search_store_sources"));
 	self->priv->search_sources_list = GTK_WIDGET (gtk_builder_get_object (builder, "gw_search_select_source"));
 	self->priv->search_results_view = GTK_WIDGET (gtk_builder_get_object (builder, "gw_search_results_view"));
 	self->priv->search_entry =  GTK_WIDGET (gtk_builder_get_object (builder, "gw_search_text"));
 
-	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (self->priv->search_sources_model),
-	                                      SEARCH_MODEL_SOURCES_NAME,
-	                                      GTK_SORT_ASCENDING);
-
-	gtk_widget_set_sensitive (self->priv->search_entry, FALSE);
-
-	g_signal_connect (self->priv->search_sources_list,
-			  "changed",
-			  G_CALLBACK (search_source_changed_cb), self);
-
 	g_signal_connect (self->priv->search_results_view,
 	                  "item-activated",
 	                  G_CALLBACK (search_activated_cb),
@@ -1250,24 +1206,12 @@ setup_sidebar_search (TotemGriloPlugin *self,
 	g_signal_connect (self->priv->search_results_view,
 	                  "button-press-event",
 	                  G_CALLBACK (context_button_pressed_cb), self);
-	g_signal_connect (self->priv->search_entry,
-	                  "changed",
-	                  G_CALLBACK (search_entry_changed_cb), self);
 
-	g_signal_connect (self->priv->search_entry,
-	                  "activate",
+	g_signal_connect (self->priv->search_entry, "activate",
 	                  G_CALLBACK (search_entry_activate_cb),
 	                  self);
-	g_signal_connect (self->priv->search_entry,
-	                  "icon-release",
-	                  G_CALLBACK (search_entry_clear_cb),
-	                  self);
-
-	g_object_set (G_OBJECT (self->priv->search_entry),
-	              "secondary-icon-name", "edit-find-symbolic",
-	              "secondary-icon-activatable", FALSE,
-	              "secondary-icon-sensitive", FALSE,
-	              NULL);
+	g_signal_connect (self->priv->search_entry, "notify::selected-id",
+			  G_CALLBACK (search_entry_source_changed_cb), self);
 
 	g_signal_connect (gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (gtk_builder_get_object (builder,
 		                    "gw_search_results_window"))),
@@ -1291,8 +1235,7 @@ add_to_pls_cb (GtkAction *action, TotemGriloPlugin *self)
 {
 	totem_add_to_playlist_and_play (self->priv->totem,
 	                                grl_media_get_url (self->priv->selected_media),
-	                                grl_media_get_title (self->priv->selected_media),
-	                                TRUE);
+	                                grl_media_get_title (self->priv->selected_media));
 }
 
 static void
@@ -1353,13 +1296,13 @@ static void
 setup_config (TotemGriloPlugin *self)
 {
 	gchar *config_file;
-	GrlPluginRegistry *registry = grl_plugin_registry_get_default ();
+	GrlRegistry *registry = grl_registry_get_default ();
 
 	/* Setup system-wide plugins configuration */
 	config_file = totem_plugin_find_file ("grilo", TOTEM_GRILO_CONFIG_FILE);
 
 	if (g_file_test (config_file, G_FILE_TEST_EXISTS)) {
-		grl_plugin_registry_add_config_from_file (registry, config_file, NULL);
+		grl_registry_add_config_from_file (registry, config_file, NULL);
 	}
 	g_free (config_file);
 
@@ -1371,7 +1314,7 @@ setup_config (TotemGriloPlugin *self)
 	                            NULL);
 
 	if (g_file_test (config_file, G_FILE_TEST_EXISTS)) {
-		grl_plugin_registry_add_config_from_file (registry, config_file, NULL);
+		grl_registry_add_config_from_file (registry, config_file, NULL);
 	}
 	g_free (config_file);
 }
@@ -1405,21 +1348,21 @@ impl_deactivate (PeasActivatable *plugin)
 	TotemGriloPlugin *self = TOTEM_GRILO_PLUGIN (plugin);
 	GList *sources;
 	GList *s;
-	GrlPluginRegistry *registry;
+	GrlRegistry *registry;
 
 	totem_remove_sidebar_page (self->priv->totem, "grilo-browse");
 	totem_remove_sidebar_page (self->priv->totem, "grilo-search");
 
-	registry = grl_plugin_registry_get_default ();
+	registry = grl_registry_get_default ();
 	g_signal_handlers_disconnect_by_func (registry, source_added_cb, self);
 	g_signal_handlers_disconnect_by_func (registry, source_removed_cb, self);
 
 	/* Shutdown all sources */
-	sources  = grl_plugin_registry_get_sources (registry, FALSE);
+	sources  = grl_registry_get_sources (registry, FALSE);
 	for (s = sources; s; s = g_list_next (s)) {
-		grl_plugin_registry_unregister_source (registry,
-		                                       GRL_MEDIA_PLUGIN (s->data),
-		                                       NULL);
+		grl_registry_unregister_source (registry,
+		                                GRL_SOURCE (s->data),
+		                                NULL);
 	}
 	g_list_free (sources);
 
