@@ -27,6 +27,8 @@
 
 #include "config.h"
 
+#define GST_USE_UNSTABLE_API 1
+
 #include <glib/gstdio.h>
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
@@ -44,6 +46,8 @@
 #include <sys/stat.h>
 
 #include "gst/totem-gst-helpers.h"
+#include "gst/totem-time-helpers.h"
+#include "gst/totem-gst-pixbuf-helpers.h"
 #include "video-utils.h"
 #include "totem-resources.h"
 
@@ -62,7 +66,7 @@
 #define GALLERY_MIN 3				/* minimum number of screenshots in a gallery */
 #define GALLERY_MAX 30				/* maximum number of screenshots in a gallery */
 #define GALLERY_HEADER_HEIGHT 66		/* header height (in pixels) for the gallery */
-#define DEFAULT_OUTPUT_SIZE 128
+#define DEFAULT_OUTPUT_SIZE 256
 
 static gboolean jpeg_output = FALSE;
 static gboolean raw_output = FALSE;
@@ -88,22 +92,23 @@ static void save_pixbuf (GdkPixbuf *pixbuf, const char *path,
 static char *
 get_special_url (GFile *file)
 {
-	char *path, *uri, *mime_type;
+	char *path, *orig_uri, *uri, *mime_type;
 	TotemDiscMediaType type;
 
 	path = g_file_get_path (file);
 
 	mime_type = g_content_type_guess (path, NULL, 0, NULL);
+	g_free (path);
 	if (g_strcmp0 (mime_type, "application/x-cd-image") != 0) {
-		g_free (path);
 		g_free (mime_type);
 		return NULL;
 	}
 	g_free (mime_type);
 
 	uri = NULL;
-	type = totem_cd_detect_type_with_url (path, &uri, NULL);
-	g_free (path);
+	orig_uri = g_file_get_uri (file);
+	type = totem_cd_detect_type_with_url (orig_uri, &uri, NULL);
+	g_free (orig_uri);
 
 	if (type == MEDIA_TYPE_DVD ||
 	    type == MEDIA_TYPE_VCD)
@@ -162,32 +167,6 @@ error_handler (GstBus *bus,
 	case GST_MESSAGE_EOS:
 		exit (0);
 
-	case GST_MESSAGE_UNKNOWN:
-	case GST_MESSAGE_WARNING:
-	case GST_MESSAGE_INFO:
-	case GST_MESSAGE_TAG:
-	case GST_MESSAGE_BUFFERING:
-	case GST_MESSAGE_STATE_CHANGED:
-	case GST_MESSAGE_STATE_DIRTY:
-	case GST_MESSAGE_STEP_DONE:
-	case GST_MESSAGE_CLOCK_PROVIDE:
-	case GST_MESSAGE_CLOCK_LOST:
-	case GST_MESSAGE_NEW_CLOCK:
-	case GST_MESSAGE_STRUCTURE_CHANGE:
-	case GST_MESSAGE_STREAM_STATUS:
-	case GST_MESSAGE_APPLICATION:
-	case GST_MESSAGE_ELEMENT:
-	case GST_MESSAGE_SEGMENT_START:
-	case GST_MESSAGE_SEGMENT_DONE:
-	case GST_MESSAGE_DURATION:
-	case GST_MESSAGE_LATENCY:
-	case GST_MESSAGE_ASYNC_START:
-	case GST_MESSAGE_ASYNC_DONE:
-	case GST_MESSAGE_REQUEST_STATE:
-	case GST_MESSAGE_STEP_START:
-	case GST_MESSAGE_QOS:
-	case GST_MESSAGE_PROGRESS:
-	case GST_MESSAGE_ANY:
 	default:
 		/* Ignored */
 		;;
@@ -210,7 +189,7 @@ thumb_app_set_error_handler (ThumbApp *app)
 	GstBus *bus;
 
 	bus = gst_element_get_bus (app->play);
-	gst_bus_set_sync_handler (bus, (GstBusSyncHandler) error_handler, app->play);
+	gst_bus_set_sync_handler (bus, (GstBusSyncHandler) error_handler, app->play, NULL);
 }
 
 static void
@@ -242,6 +221,7 @@ check_cover_for_stream (ThumbApp   *app,
 static void
 thumb_app_check_for_cover (ThumbApp *app)
 {
+	PROGRESS_DEBUG ("Checking whether file has cover");
 	check_cover_for_stream (app, "get-audio-tags");
 	check_cover_for_stream (app, "get-video-tags");
 }
@@ -249,16 +229,25 @@ thumb_app_check_for_cover (ThumbApp *app)
 static gboolean
 thumb_app_set_duration (ThumbApp *app)
 {
-	GstFormat fmt = GST_FORMAT_TIME;
 	gint64 len = -1;
 
-	if (gst_element_query_duration (app->play, &fmt, &len) && len != -1) {
+	if (gst_element_query_duration (app->play, GST_FORMAT_TIME, &len) && len != -1) {
 		app->duration = len / GST_MSECOND;
 		return TRUE;
 	}
+	app->duration = -1;
 	return FALSE;
 }
 
+static void
+assert_duration (ThumbApp *app)
+{
+	if (app->duration != -1)
+		return;
+	g_print ("totem-video-thumbnailer couldn't get the duration of file '%s'\n", app->input);
+	exit (1);
+}
+
 static gboolean
 thumb_app_get_has_video (ThumbApp *app)
 {
@@ -296,32 +285,6 @@ thumb_app_start (ThumbApp *app)
 			totem_gst_message_print (message, app->play, "totem-video-thumbnailer-error");
 			return FALSE;
 
-		case GST_MESSAGE_UNKNOWN:
-		case GST_MESSAGE_EOS:
-		case GST_MESSAGE_WARNING:
-		case GST_MESSAGE_INFO:
-		case GST_MESSAGE_TAG:
-		case GST_MESSAGE_BUFFERING:
-		case GST_MESSAGE_STATE_CHANGED:
-		case GST_MESSAGE_STATE_DIRTY:
-		case GST_MESSAGE_STEP_DONE:
-		case GST_MESSAGE_CLOCK_PROVIDE:
-		case GST_MESSAGE_CLOCK_LOST:
-		case GST_MESSAGE_NEW_CLOCK:
-		case GST_MESSAGE_STRUCTURE_CHANGE:
-		case GST_MESSAGE_STREAM_STATUS:
-		case GST_MESSAGE_APPLICATION:
-		case GST_MESSAGE_ELEMENT:
-		case GST_MESSAGE_SEGMENT_START:
-		case GST_MESSAGE_SEGMENT_DONE:
-		case GST_MESSAGE_DURATION:
-		case GST_MESSAGE_LATENCY:
-		case GST_MESSAGE_ASYNC_START:
-		case GST_MESSAGE_REQUEST_STATE:
-		case GST_MESSAGE_STEP_START:
-		case GST_MESSAGE_QOS:
-		case GST_MESSAGE_PROGRESS:
-		case GST_MESSAGE_ANY:
 		default:
 			/* Ignore */
 			;;
@@ -344,7 +307,7 @@ thumb_app_setup_play (ThumbApp *app)
 	GstElement *play;
 	GstElement *audio_sink, *video_sink;
 
-	play = gst_element_factory_make ("playbin2", "play");
+	play = gst_element_factory_make ("playbin", "play");
 	audio_sink = gst_element_factory_make ("fakesink", "audio-fake-sink");
 	video_sink = gst_element_factory_make ("fakesink", "video-fake-sink");
 	g_object_set (video_sink, "sync", TRUE, NULL);
@@ -641,6 +604,16 @@ save_pixbuf (GdkPixbuf *pixbuf, const char *path,
 	g_object_unref (with_holes);
 }
 
+static GdkPixbuf *
+capture_frame_at_time (ThumbApp   *app,
+		       gint64 milliseconds)
+{
+	if (milliseconds != 0)
+		thumb_app_seek (app, milliseconds);
+
+	return totem_gst_playbin_get_frame (app->play);
+}
+
 static GdkPixbuf *
 capture_interesting_frame (ThumbApp *app)
 {
@@ -654,6 +627,11 @@ capture_interesting_frame (ThumbApp *app)
 		0.5
 	};
 
+	if (app->duration == -1) {
+		PROGRESS_DEBUG("Video has no duration, so capture 1st frame");
+		return capture_frame_at_time (app, 0);
+	}
+
 	/* Test at multiple points in the file to see if we can get an
 	 * interesting frame */
 	for (current = 0; current < G_N_ELEMENTS(frame_locations); current++)
@@ -682,15 +660,6 @@ capture_interesting_frame (ThumbApp *app)
 	return pixbuf;
 }
 
-static GdkPixbuf *
-capture_frame_at_time (ThumbApp   *app,
-		       gint64 milliseconds)
-{
-	thumb_app_seek (app, milliseconds);
-
-	return totem_gst_playbin_get_frame (app->play);
-}
-
 static GdkPixbuf *
 cairo_surface_to_pixbuf (cairo_surface_t *surface)
 {
@@ -978,10 +947,6 @@ int main (int argc, char *argv[])
 	const char *input, *output;
 	ThumbApp app;
 
-#if !GLIB_CHECK_VERSION (2, 31, 0)
-	g_thread_init (NULL);
-#endif
-
 	context = g_option_context_new ("Thumbnail movies");
 	options = gst_init_get_option_group ();
 	g_option_context_add_main_entries (context, entries, GETTEXT_PACKAGE);
@@ -1057,13 +1022,10 @@ int main (int argc, char *argv[])
 	if (gallery == -1)
 		thumb_app_check_for_cover (&app);
 	if (thumb_app_get_has_video (&app) == FALSE) {
-		g_debug ("totem-video-thumbnailer couldn't find a video track in '%s'\n", input);
-		exit (1);
-	}
-	if (thumb_app_set_duration (&app) == FALSE) {
-		g_print ("totem-video-thumbnailer couldn't get the duration of file '%s'\n", input);
+		PROGRESS_DEBUG ("totem-video-thumbnailer couldn't find a video track in '%s'\n", input);
 		exit (1);
 	}
+	thumb_app_set_duration (&app);
 
 	PROGRESS_DEBUG("Opened video file: '%s'", input);
 	PRINT_PROGRESS (10.0);
@@ -1072,12 +1034,15 @@ int main (int argc, char *argv[])
 		/* If the user has told us to use a frame at a specific second
 		 * into the video, just use that frame no matter how boring it
 		 * is */
-		if (second_index != -1)
+		if (second_index != -1) {
+			assert_duration (&app);
 			pixbuf = capture_frame_at_time (&app, second_index * 1000);
-		else
+		} else {
 			pixbuf = capture_interesting_frame (&app);
+		}
 		PRINT_PROGRESS (90.0);
 	} else {
+		assert_duration (&app);
 		/* We're producing a gallery of screenshots from throughout the file */
 		pixbuf = create_gallery (&app);
 	}
